@@ -12,8 +12,8 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/AABB_tree/include/CGAL/AABB_tree.h $
+// $Id: AABB_tree.h 11bf392 2019-02-16T14:00:03+01:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0+
 //
 //
@@ -44,7 +44,7 @@
 
 namespace CGAL {
 
-/// \addtogroup PkgAABB_tree
+/// \addtogroup PkgAABBTreeRef
 /// @{
 
 	/**
@@ -156,7 +156,29 @@ namespace CGAL {
     /// `build()` explicitly to ensure that the next call to
     /// query functions will not trigger the reconstruction of the
     /// data structure.
+    /// A call to `AABBTraits::set_shared_data(t...)`
+    /// is made using the internally stored traits.
+    /// For compilers that do not support variadic templates,
+    /// overloads up to 5 template arguments are provided.
+#if !defined(CGAL_CFG_NO_CPP0X_VARIADIC_TEMPLATES) && !defined(CGAL_CFG_NO_CPP0X_RVALUE_REFERENCE)
+    template<typename ... T>
+    void build(T&& ...);
+#ifndef DOXYGEN_RUNNING
     void build();
+#endif
+#else
+    void build();
+    template<typename T1>
+    void build(T1& t1);
+    template<typename T1, typename T2>
+    void build(T1& t1, T2& t2);
+    template<typename T1, typename T2,typename T3>
+    void build(T1& t1, T2& t2, T3& t3);
+    template<typename T1, typename T2,typename T3,typename T4>
+    void build(T1& t1, T2& t2, T3& t3, T4& t4);
+    template<typename T1, typename T2,typename T3,typename T4,typename T5>
+    void build(T1& t1, T2& t2, T3& t3, T4& t4, T5& t5);
+#endif
     ///@}
 
 		/// \name Operations
@@ -407,6 +429,12 @@ public:
     /// that returns `true` in order to skip the primitive.
     /// Defaults to a functor that always returns `false`.
     ///
+    /// \note `skip` might be given some primitives that are not intersected by `query`
+    ///       because the intersection test is done after the skip test. Also note that
+    ///       the order the primitives are given to `skip` is not necessarily the
+    ///       intersection order with `query`.
+    ///
+    ///
     /// `AABBTraits` must be a model of `AABBRayIntersectionTraits` to
     /// call this member function.
     template<typename Ray, typename SkipFunctor>
@@ -646,6 +674,18 @@ public:
 			else
 				return this->any_reference_point_and_id();
 		}
+		
+		//! Returns the datum (geometric object) represented `p`. 
+#ifndef DOXYGEN_RUNNING
+		typename internal::Primitive_helper<AABBTraits>::Datum_type 
+#else
+		typename AABBTraits::Primitive::Datum_reference 
+#endif
+		datum(Primitive& p)const
+		{
+		  return internal::Primitive_helper<AABBTraits>::
+		      get_datum(p, this->traits());
+		}
 
 	private:
     //Traits class
@@ -728,7 +768,7 @@ public:
                              ConstPrimitiveIterator beyond,
                              T&& ... t)
 	{
-    set_shared_data(std::forward<T>(t)...);
+		set_shared_data(std::forward<T>(t)...);
 		while(first != beyond)
 		{
 			m_primitives.push_back(Primitive(first,std::forward<T>(t)...));
@@ -752,6 +792,14 @@ public:
 
     build();
 	}  
+        
+        template<typename Tr>
+        template<typename ... T>
+        void AABB_tree<Tr>::build(T&& ... t)
+        {
+          set_shared_data(std::forward<T>(t)...);
+          build();
+        }
   #else
   //=============constructor======================
 	template<typename Tr>
@@ -1033,7 +1081,43 @@ public:
 
     build();
 	}
-  #endif
+        //overloads calling set_shared_data()
+        template<typename Tr>
+        template<typename T1>
+        void AABB_tree<Tr>::build(T1& t1)
+        {
+          set_shared_data(t1);
+          build();
+        }
+        template<typename Tr>
+        template<typename T1, typename T2>
+        void AABB_tree<Tr>::build(T1& t1, T2& t2)
+        {
+          set_shared_data(t1, t2);
+          build();
+        }
+        template<typename Tr>
+        template<typename T1, typename T2, typename T3>
+        void AABB_tree<Tr>::build(T1& t1, T2& t2, T3& t3)
+        {
+          set_shared_data(t1, t2, t3);
+          build();
+        }
+        template<typename Tr>
+        template<typename T1, typename T2, typename T3, typename T4>
+        void AABB_tree<Tr>::build(T1& t1, T2& t2, T3& t3, T4& t4)
+        {
+          set_shared_data(t1, t2, t3, t4);
+          build();
+        }
+        template<typename Tr>
+        template<typename T1, typename T2, typename T3, typename T4, typename T5>
+        void AABB_tree<Tr>::build(T1& t1, T2& t2, T3& t3, T4& t4, T5& t5)
+        {
+          set_shared_data(t1, t2, t3, t4, t5);
+          build();
+        }
+#endif
 
 	template<typename Tr>
 	void AABB_tree<Tr>::insert(const Primitive& p)
@@ -1062,7 +1146,7 @@ public:
 
 			// constructs the tree
 			m_p_root_node->expand(m_primitives.begin(), m_primitives.end(),
-														m_primitives.size(), m_traits);
+					      m_primitives.size(), m_traits);
 		}
 
 
