@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Nef_2/include/CGAL/Extended_homogeneous.h $
+// $Id: Extended_homogeneous.h 97b5c37 2018-03-04T17:33:50+00:00 Andreas Fabri
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Michael Seel <seel@mpi-sb.mpg.de>
@@ -22,11 +23,12 @@
 
 #include <CGAL/license/Nef_2.h>
 
+#include <CGAL/disable_warnings.h>
 
 #include <CGAL/basic.h>
 #include <CGAL/Homogeneous.h> 
 #include <CGAL/Point_2.h> 
-#include <CGAL/Line_2_Line_2_intersection.h> 
+#include <CGAL/Intersections_2/Line_2_Line_2.h> 
 #include <CGAL/squared_distance_2.h> 
 #include <CGAL/number_utils.h>
 #include <CGAL/Nef_polynomial.h>
@@ -519,4 +521,7 @@ on the extended geometric objects.}*/
 };
 
 } //namespace CGAL
+
+#include <CGAL/enable_warnings.h>
+
 #endif // CGAL_EXTENDED_HOMOGENEOUS_H
