@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Nef_S2/include/CGAL/Nef_S2/Generic_handle_map.h $
+// $Id: Generic_handle_map.h ee57fc2 2017-10-21T01:03:14+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Michael Seel  <seel@mpi-sb.mpg.de>
@@ -21,6 +22,9 @@
 #ifndef CGAL_GENERIC_HANDLE_MAP_H
 #define CGAL_GENERIC_HANDLE_MAP_H
 
+#include <CGAL/license/Nef_S2.h>
+
+
 #include <CGAL/Unique_hash_map.h>
 
 namespace CGAL {
