@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Point_set_3/include/CGAL/Point_set_3/IO.h $
+// $Id: IO.h 2402dcf 2018-12-14T15:10:48+01:00 Simon Giraudot
+// SPDX-License-Identifier: GPL-3.0+
 //
 //
 // Author(s)     : Simon Giraudot
@@ -23,6 +24,7 @@
 
 #include <CGAL/license/Point_set_3.h>
 
+#include <CGAL/Point_set_3.h>
 #include <CGAL/IO/read_xyz_points.h>
 #include <CGAL/IO/read_off_points.h>
 #include <CGAL/IO/write_xyz_points.h>
@@ -43,6 +45,80 @@ namespace CGAL {
 namespace internal
 {
 
+  template <typename Point, typename Vector>
+  class Abstract_property_printer
+  {
+  public:
+    virtual ~Abstract_property_printer() { }
+    virtual void print (std::ostream& stream, const typename CGAL::Point_set_3<Point,Vector>::Index& index) = 0;
+  };
+
+  template <typename Point, typename Vector, typename Type>
+  class Property_printer : public Abstract_property_printer<Point, Vector>
+  {
+    typedef typename CGAL::Point_set_3<Point, Vector> Point_set;
+    typedef typename Point_set::template Property_map<Type> Pmap;
+    Pmap m_pmap;
+  public:
+    Property_printer (const Pmap& pmap) : m_pmap (pmap)
+    {
+
+    }
+    
+    virtual void print(std::ostream& stream, const typename CGAL::Point_set_3<Point,Vector>::Index& index)
+    {
+      stream << get(m_pmap, index);
+    }
+  };
+
+  template <typename Point, typename Vector, typename Type>
+  class Simple_property_printer : public Abstract_property_printer<Point, Vector>
+  {
+    typedef typename CGAL::Point_set_3<Point, Vector> Point_set;
+    typedef typename Point_set::template Property_map<Type> Pmap;
+    Pmap m_pmap;
+  public:
+    Simple_property_printer (const Pmap& pmap) : m_pmap (pmap)
+    {
+
+    }
+    
+    virtual void print(std::ostream& stream, const typename CGAL::Point_set_3<Point,Vector>::Index& index)
+    {
+      if (get_mode(stream) == IO::ASCII)
+        stream << get(m_pmap, index);
+      else
+        {
+          Type t = get (m_pmap, index);
+          stream.write (reinterpret_cast<char*>(&t), sizeof(t));
+        }
+    }
+  };
+
+  template <typename Point, typename Vector, typename Type>
+  class Char_property_printer : public Abstract_property_printer<Point, Vector>
+  {
+    typedef typename CGAL::Point_set_3<Point, Vector> Point_set;
+    typedef typename Point_set::template Property_map<Type> Pmap;
+    Pmap m_pmap;
+  public:
+    Char_property_printer (const Pmap& pmap) : m_pmap (pmap)
+    {
+
+    }
+    
+    virtual void print(std::ostream& stream, const typename CGAL::Point_set_3<Point,Vector>::Index& index)
+    {
+      if (get_mode(stream) == IO::ASCII)
+        stream << int(get(m_pmap, index));
+      else
+        {
+          Type t = get (m_pmap, index);
+          stream.write (reinterpret_cast<char*>(&t), sizeof(t));
+        }
+    }
+  };
+  
 #if !defined(CGAL_CFG_NO_CPP0X_RVALUE_REFERENCE) && !defined(CGAL_CFG_NO_CPP0X_VARIADIC_TEMPLATES)
 namespace PLY
 {
@@ -59,7 +135,7 @@ private:
   struct Abstract_ply_property_to_point_set_property
   {
     virtual ~Abstract_ply_property_to_point_set_property() { }
-    virtual void assign (PLY_reader& reader, typename Point_set::Index index) = 0;
+    virtual void assign (PLY_element& element, typename Point_set::Index index) = 0;
   };
 
   template <typename Type>
@@ -78,10 +154,10 @@ private:
       m_pmap = ps.push_property_map (m_map);
     }
     
-    virtual void assign (PLY_reader& reader, typename Point_set::Index index)
+    virtual void assign (PLY_element& element, typename Point_set::Index index)
     {
-      Type t;
-      reader.assign (t, m_name.c_str());
+      Type t{};
+      element.assign (t, m_name.c_str());
       put(m_pmap, index, t);
     }
   };
@@ -96,123 +172,127 @@ public:
     : m_point_set (point_set), m_use_floats (false)
   { }
 
-  void instantiate_properties  (PLY_reader& reader)
+  ~Point_set_3_filler()
   {
-    const std::vector<PLY_read_number*>& readers
-      = reader.readers();
+    for (std::size_t i = 0; i < m_properties.size(); ++ i)
+      delete m_properties[i];
+  }
 
+  void instantiate_properties  (PLY_element& element)
+  {
     bool has_normal[3] = { false, false, false };
     
-    for (std::size_t i = 0; i < readers.size(); ++ i)
+    for (std::size_t j = 0; j < element.number_of_properties(); ++ j)
+    {
+      internal::PLY::PLY_read_number* property = element.property(j);
+        
+      const std::string& name = property->name();
+      if (name == "x" ||
+          name == "y" ||
+          name == "z")
       {
-        const std::string& name = readers[i]->name();
-        if (name == "x" ||
-            name == "y" ||
-            name == "z")
-          {
-            if (dynamic_cast<PLY_read_typed_number<float>*>(readers[i]))
-              m_use_floats = true;
-            continue;
-          }
-        if (name == "nx")
-          {
-            has_normal[0] = true;
-            continue;
-          }
-        if (name == "ny")
-          {
-            has_normal[1] = true;
-            continue;
-          }
-        if (name == "nz")
-          {
-            has_normal[2] = true;
-            continue;
-          }
-
-        if (dynamic_cast<PLY_read_typed_number<boost::int8_t>*>(readers[i]))
-          {
-            m_properties.push_back
-              (new PLY_property_to_point_set_property<boost::int8_t>(m_point_set,
-                                                                     name));
-          }
-        else if (dynamic_cast<PLY_read_typed_number<boost::uint8_t>*>(readers[i]))
-          {
-            m_properties.push_back
-              (new PLY_property_to_point_set_property<boost::uint8_t>(m_point_set,
-                                                                      name));
-          }
-        else if (dynamic_cast<PLY_read_typed_number<boost::int16_t>*>(readers[i]))
-          {
-            m_properties.push_back
-              (new PLY_property_to_point_set_property<boost::int16_t>(m_point_set,
-                                                                      name));
-          }
-        else if (dynamic_cast<PLY_read_typed_number<boost::uint16_t>*>(readers[i]))
-          {
-            m_properties.push_back
-              (new PLY_property_to_point_set_property<boost::uint16_t>(m_point_set,
-                                                                       name));
-          }
-        else if (dynamic_cast<PLY_read_typed_number<boost::int32_t>*>(readers[i]))
-          {
-            m_properties.push_back
-              (new PLY_property_to_point_set_property<boost::int32_t>(m_point_set,
-                                                                      name));
-          }
-        else if (dynamic_cast<PLY_read_typed_number<boost::uint32_t>*>(readers[i]))
-          {
-            m_properties.push_back
-              (new PLY_property_to_point_set_property<boost::uint32_t>(m_point_set,
-                                                                       name));
-          }
-        else if (dynamic_cast<PLY_read_typed_number<float>*>(readers[i]))
-          {
-            m_properties.push_back
-              (new PLY_property_to_point_set_property<float>(m_point_set,
-                                                             name));
-          }
-        else if (dynamic_cast<PLY_read_typed_number<double>*>(readers[i]))
-          {
-            m_properties.push_back
-              (new PLY_property_to_point_set_property<double>(m_point_set,
-                                                             name));
-          }
+        if (dynamic_cast<PLY_read_typed_number<float>*>(property))
+          m_use_floats = true;
+        continue;
+      }
+      if (name == "nx")
+      {
+        has_normal[0] = true;
+        continue;
+      }
+      if (name == "ny")
+      {
+        has_normal[1] = true;
+        continue;
+      }
+      if (name == "nz")
+      {
+        has_normal[2] = true;
+        continue;
       }
 
+      if (dynamic_cast<PLY_read_typed_number<boost::int8_t>*>(property))
+      {
+        m_properties.push_back
+          (new PLY_property_to_point_set_property<boost::int8_t>(m_point_set,
+                                                                 name));
+      }
+      else if (dynamic_cast<PLY_read_typed_number<boost::uint8_t>*>(property))
+      {
+        m_properties.push_back
+          (new PLY_property_to_point_set_property<boost::uint8_t>(m_point_set,
+                                                                  name));
+      }
+      else if (dynamic_cast<PLY_read_typed_number<boost::int16_t>*>(property))
+      {
+        m_properties.push_back
+          (new PLY_property_to_point_set_property<boost::int16_t>(m_point_set,
+                                                                  name));
+      }
+      else if (dynamic_cast<PLY_read_typed_number<boost::uint16_t>*>(property))
+      {
+        m_properties.push_back
+          (new PLY_property_to_point_set_property<boost::uint16_t>(m_point_set,
+                                                                   name));
+      }
+      else if (dynamic_cast<PLY_read_typed_number<boost::int32_t>*>(property))
+      {
+        m_properties.push_back
+          (new PLY_property_to_point_set_property<boost::int32_t>(m_point_set,
+                                                                  name));
+      }
+      else if (dynamic_cast<PLY_read_typed_number<boost::uint32_t>*>(property))
+      {
+        m_properties.push_back
+          (new PLY_property_to_point_set_property<boost::uint32_t>(m_point_set,
+                                                                   name));
+      }
+      else if (dynamic_cast<PLY_read_typed_number<float>*>(property))
+      {
+        m_properties.push_back
+          (new PLY_property_to_point_set_property<float>(m_point_set,
+                                                         name));
+      }
+      else if (dynamic_cast<PLY_read_typed_number<double>*>(property))
+      {
+        m_properties.push_back
+          (new PLY_property_to_point_set_property<double>(m_point_set,
+                                                          name));
+      }
+    }
     if (has_normal[0] && has_normal[1] && has_normal[2])
       m_point_set.add_normal_map();
   }
   
-  void process_line (PLY_reader& reader)
+  void process_line (PLY_element& element)
   {
     m_point_set.insert();
     
     if (m_use_floats)
-      process_line<float>(reader);
+      process_line<float>(element);
     else
-      process_line<double>(reader);
+      process_line<double>(element);
 
     for (std::size_t i = 0; i < m_properties.size(); ++ i)
-      m_properties[i]->assign (reader, *(m_point_set.end() - 1));
+      m_properties[i]->assign (element, *(m_point_set.end() - 1));
   }
 
   template <typename FT>
-  void process_line (PLY_reader& reader)
+  void process_line (PLY_element& element)
   {
     FT x = (FT)0.,y = (FT)0., z = (FT)0.,
       nx = (FT)0., ny = (FT)0., nz = (FT)0.;
-    reader.assign (x, "x");
-    reader.assign (y, "y");
-    reader.assign (z, "z");
+    element.assign (x, "x");
+    element.assign (y, "y");
+    element.assign (z, "z");
     Point point (x, y, z);
     m_point_set.point(*(m_point_set.end() - 1)) = point;
 
     if (m_point_set.has_normal_map())
       {
-        reader.assign (nx, "nx");
-        reader.assign (ny, "ny");
-        reader.assign (nz, "nz");
+        element.assign (nx, "nx");
+        element.assign (ny, "ny");
+        element.assign (nz, "nz");
         Vector normal (nx, ny, nz);
         m_point_set.normal(*(m_point_set.end() - 1)) = normal;
       }
@@ -235,11 +315,11 @@ read_xyz_point_set(
 {
   point_set.add_normal_map();
 
-  bool out = CGAL::read_xyz_points_and_normals
+  bool out = CGAL::read_xyz_points
     (stream,
      point_set.index_back_inserter(),
-     point_set.point_push_map(),
-     point_set.normal_push_map());
+     CGAL::parameters::point_map(point_set.point_push_map()).
+     normal_map(point_set.normal_push_map()));
 
   bool has_normals = false;
   for (typename CGAL::Point_set_3<Point, Vector>::const_iterator it = point_set.begin();
@@ -267,11 +347,11 @@ read_off_point_set(
 {
   point_set.add_normal_map();
 
-  bool out = CGAL::read_off_points_and_normals
+  bool out = CGAL::read_off_points
     (stream,
      point_set.index_back_inserter(),
-     point_set.point_push_map(),
-     point_set.normal_push_map());
+     CGAL::parameters::point_map(point_set.point_push_map()).
+     normal_map(point_set.normal_push_map()));
 
   bool has_normals = false;
   for (typename CGAL::Point_set_3<Point, Vector>::const_iterator it = point_set.begin();
@@ -297,7 +377,12 @@ template <typename Point, typename Vector>
 bool
 read_ply_point_set(
   std::istream& stream, ///< input stream.
+#ifdef DOXYGEN_RUNNING
   CGAL::Point_set_3<Point, Vector>& point_set) ///< point set
+#else
+  CGAL::Point_set_3<Point, Vector>& point_set, ///< point set
+  std::string* comments = NULL) ///< recover PLY comments
+#endif
 {
   if(!stream)
     {
@@ -309,26 +394,42 @@ read_ply_point_set(
   internal::PLY::Point_set_3_filler<Point, Vector> filler(point_set);
   
   if (!(reader.init (stream)))
+  {
+    stream.setstate(std::ios::failbit);
     return false;
+  }
 
-  filler.instantiate_properties (reader);
-  
-  std::size_t points_read = 0;
-  
-  while (!(stream.eof()) && points_read < reader.m_nb_points)
+  if (comments != NULL)
+    *comments = reader.comments();
+
+  for (std::size_t i = 0; i < reader.number_of_elements(); ++ i)
+  {
+    internal::PLY::PLY_element& element = reader.element(i);
+
+    if (element.name() == "vertex" || element.name() == "vertices")
     {
-      for (std::size_t i = 0; i < reader.readers().size (); ++ i)
-        reader.readers()[i]->get (stream);
-
-      filler.process_line (reader);
-      
-      ++ points_read;
+      point_set.reserve (element.number_of_items());
+      filler.instantiate_properties (element);
     }
-  // Skip remaining lines
+  
+    for (std::size_t j = 0; j < element.number_of_items(); ++ j)
+    {
+      for (std::size_t k = 0; k < element.number_of_properties(); ++ k)
+      {
+        internal::PLY::PLY_read_number* property = element.property(k);
+        property->get (stream);
+        if (stream.fail())
+          return false;
+      }
 
-  return (points_read == reader.m_nb_points);
+      if (element.name() == "vertex" || element.name() == "vertices")
+        filler.process_line (element);
+    }
+  }
+
+  return true;
 }
-
+  
 /*!
   \ingroup PkgPointSet3IO
  */
@@ -336,10 +437,155 @@ template <typename Point, typename Vector>
 bool
 write_ply_point_set(
   std::ostream& stream, ///< output stream.
+#ifdef DOXYGEN_RUNNING
   const CGAL::Point_set_3<Point, Vector>& point_set)  ///< point set
+#else
+  const CGAL::Point_set_3<Point, Vector>& point_set,  ///< point set
+  std::string* comments = NULL) ///< write PLY comments
+#endif
 {
+  typedef CGAL::Point_set_3<Point, Vector> Point_set;
+    
+  stream << "ply" << std::endl
+         << ((get_mode(stream) == IO::BINARY) ? "format binary_little_endian 1.0" : "format ascii 1.0") << std::endl
+         << "comment Generated by the CGAL library" << std::endl;
 
-  stream << point_set;
+  if (comments != NULL)
+  {
+    std::istringstream iss (*comments);
+    std::string line;
+    while (getline(iss, line))
+    {
+      if (line != "Generated by the CGAL library") // Avoid repeating the line if multiple savings
+        stream << "comment " << line << std::endl;
+    }
+  }
+  
+  stream << "element vertex " << point_set.number_of_points() << std::endl;
+  
+  std::vector<std::string> prop = point_set.base().properties();
+  std::vector<internal::Abstract_property_printer<Point, Vector>*> printers;
+  
+  for (std::size_t i = 0; i < prop.size(); ++ i)
+    {
+      if (prop[i] == "index")
+        continue;
+
+      if (prop[i] == "point")
+        {
+          if (boost::is_same<typename GetFTFromMap<typename Point_set::Point_map>::type, float>::value)
+          {
+            stream << "property float x" << std::endl
+                   << "property float y" << std::endl
+                   << "property float z" << std::endl;
+          }
+          else
+          {
+            stream << "property double x" << std::endl
+                   << "property double y" << std::endl
+                   << "property double z" << std::endl;
+          }
+          printers.push_back (new internal::Property_printer<Point,Vector,Point>(point_set.point_map()));
+          continue;
+        }
+      if (prop[i] == "normal")
+        {
+          stream << "property double nx" << std::endl
+                 << "property double ny" << std::endl
+                 << "property double nz" << std::endl;
+          printers.push_back (new internal::Property_printer<Point,Vector,Vector>(point_set.normal_map()));
+          continue;
+        }
+      
+      bool okay = false;
+      {
+        typename Point_set::template Property_map<boost::int8_t> pmap;
+        boost::tie (pmap, okay) = point_set.template property_map<boost::int8_t>(prop[i]);
+        if (okay)
+          {
+            stream << "property char " << prop[i] << std::endl;
+            printers.push_back (new internal::Char_property_printer<Point,Vector,boost::int8_t>(pmap));
+            continue;
+          }
+      }
+      {
+        typename Point_set::template Property_map<boost::uint8_t> pmap;
+        boost::tie (pmap, okay) = point_set.template property_map<boost::uint8_t>(prop[i]);
+        if (okay)
+          {
+            stream << "property uchar " << prop[i] << std::endl;
+            printers.push_back (new internal::Char_property_printer<Point,Vector,boost::uint8_t>(pmap));
+            continue;
+          }
+      }
+      {
+        typename Point_set::template Property_map<boost::int16_t> pmap;
+        boost::tie (pmap, okay) = point_set.template property_map<boost::int16_t>(prop[i]);
+        if (okay)
+          {
+            stream << "property short " << prop[i] << std::endl;
+            printers.push_back (new internal::Simple_property_printer<Point,Vector,boost::int16_t>(pmap));
+            continue;
+          }
+      }
+      {
+        typename Point_set::template Property_map<boost::uint16_t> pmap;
+        boost::tie (pmap, okay) = point_set.template property_map<boost::uint16_t>(prop[i]);
+        if (okay)
+          {
+            stream << "property ushort " << prop[i] << std::endl;
+            printers.push_back (new internal::Simple_property_printer<Point,Vector,boost::uint16_t>(pmap));
+            continue;
+          }
+      }
+      {
+        typename Point_set::template Property_map<boost::int32_t> pmap;
+        boost::tie (pmap, okay) = point_set.template property_map<boost::int32_t>(prop[i]);
+        if (okay)
+          {
+            stream << "property int " << prop[i] << std::endl;
+            printers.push_back (new internal::Simple_property_printer<Point,Vector,boost::int32_t>(pmap));
+            continue;
+          }
+      }
+      {
+        typename Point_set::template Property_map<float> pmap;
+        boost::tie (pmap, okay) = point_set.template property_map<float>(prop[i]);
+        if (okay)
+          {
+            stream << "property float " << prop[i] << std::endl;
+            printers.push_back (new internal::Simple_property_printer<Point,Vector,float>(pmap));
+            continue;
+          }
+      }
+      {
+        typename Point_set::template Property_map<double> pmap;
+        boost::tie (pmap, okay) = point_set.template property_map<double>(prop[i]);
+        if (okay)
+          {
+            stream << "property double " << prop[i] << std::endl;
+            printers.push_back (new internal::Simple_property_printer<Point,Vector,double>(pmap));
+            continue;
+          }
+      }
+    }
+    
+  stream << "end_header" << std::endl;  
+
+  for (typename Point_set::const_iterator it = point_set.begin(); it != point_set.end(); ++ it)
+    {
+      for (std::size_t i = 0; i < printers.size(); ++ i)
+        {
+          printers[i]->print(stream, *it);
+          if (get_mode (stream) == IO::ASCII)
+            stream << " ";
+        }
+      if (get_mode (stream) == IO::ASCII)
+        stream << std::endl;
+    }
+
+  for (std::size_t i = 0; i < printers.size(); ++ i)
+    delete printers[i];
   return true;
 }
 
@@ -578,7 +824,7 @@ write_las_point_set(
   
   bool okay
     = write_las_points_with_properties
-    (stream, point_set.begin(), point_set.end(),
+    (stream, point_set,
      make_las_point_writer (point_set.point_map()),
      std::make_pair (intensity, LAS_property::Intensity()),
      std::make_pair (return_number, LAS_property::Return_number()),
@@ -629,14 +875,14 @@ struct template_tag_false : public CGAL::Tag_false
   
 template <typename Point, typename Vector>
 bool
-read_ply_point_set(std::istream&, CGAL::Point_set_3<Point, Vector>&)
+read_ply_point_set(std::istream&, CGAL::Point_set_3<Point, Vector>&, std::string* = NULL)
 {
   CGAL_static_assertion_msg (template_tag_false<Point>::value, "CGAL PLY reader requires a C++11 compiler");
   return false;
 }
 template <typename Point, typename Vector>
 bool
-write_ply_point_set(std::ostream&, const CGAL::Point_set_3<Point, Vector>&)
+write_ply_point_set(std::ostream&, const CGAL::Point_set_3<Point, Vector>&, std::string* = NULL)
 {
   CGAL_static_assertion_msg (template_tag_false<Point>::value, "CGAL PLY writer requires a C++11 compiler");
   return false;
@@ -667,13 +913,14 @@ write_xyz_point_set(
   const CGAL::Point_set_3<Point, Vector>& point_set)  ///< point set
 {
   if (point_set.has_normal_map())
-    return CGAL::write_xyz_points_and_normals
-      (stream, point_set.begin(), point_set.end(),
-       point_set.point_map(), point_set.normal_map());
+    return CGAL::write_xyz_points
+      (stream, point_set,
+       CGAL::parameters::point_map(point_set.point_map()).
+       normal_map(point_set.normal_map()));
   
   return CGAL::write_xyz_points
-  (stream, point_set.begin(), point_set.end(),
-   point_set.point_map());
+    (stream, point_set,
+     CGAL::parameters::point_map(point_set.point_map()));
 }
 
 /*!
@@ -686,13 +933,14 @@ write_off_point_set(
   const CGAL::Point_set_3<Point, Vector>& point_set)  ///< point set
 {
   if (point_set.has_normal_map())
-    return CGAL::write_off_points_and_normals
-      (stream, point_set.begin(), point_set.end(),
-       point_set.point_map(), point_set.normal_map());
+    return CGAL::write_off_points
+      (stream, point_set,
+       CGAL::parameters::point_map(point_set.point_map()).
+       normal_map(point_set.normal_map()));
   
   return CGAL::write_off_points
-  (stream, point_set.begin(), point_set.end(),
-   point_set.point_map());
+    (stream, point_set,
+     CGAL::parameters::point_map(point_set.point_map()));
 }
 
   
@@ -738,86 +986,6 @@ std::istream& operator>>(std::istream& is,
   return is;
 }
 
-/// \cond SKIP_IN_MANUAL
-namespace internal
-{
-  template <typename Point, typename Vector>
-  class Abstract_property_printer
-  {
-  public:
-    virtual ~Abstract_property_printer() { }
-    virtual void print (std::ostream& stream, const typename CGAL::Point_set_3<Point,Vector>::Index& index) = 0;
-  };
-
-  template <typename Point, typename Vector, typename Type>
-  class Property_printer : public Abstract_property_printer<Point, Vector>
-  {
-    typedef typename CGAL::Point_set_3<Point, Vector> Point_set;
-    typedef typename Point_set::template Property_map<Type> Pmap;
-    Pmap m_pmap;
-  public:
-    Property_printer (const Pmap& pmap) : m_pmap (pmap)
-    {
-
-    }
-    
-    virtual void print(std::ostream& stream, const typename CGAL::Point_set_3<Point,Vector>::Index& index)
-    {
-      stream << get(m_pmap, index);
-    }
-  };
-
-  template <typename Point, typename Vector, typename Type>
-  class Simple_property_printer : public Abstract_property_printer<Point, Vector>
-  {
-    typedef typename CGAL::Point_set_3<Point, Vector> Point_set;
-    typedef typename Point_set::template Property_map<Type> Pmap;
-    Pmap m_pmap;
-  public:
-    Simple_property_printer (const Pmap& pmap) : m_pmap (pmap)
-    {
-
-    }
-    
-    virtual void print(std::ostream& stream, const typename CGAL::Point_set_3<Point,Vector>::Index& index)
-    {
-      if (get_mode(stream) == IO::ASCII)
-        stream << get(m_pmap, index);
-      else
-        {
-          Type t = get (m_pmap, index);
-          stream.write (reinterpret_cast<char*>(&t), sizeof(t));
-        }
-    }
-  };
-
-  template <typename Point, typename Vector, typename Type>
-  class Char_property_printer : public Abstract_property_printer<Point, Vector>
-  {
-    typedef typename CGAL::Point_set_3<Point, Vector> Point_set;
-    typedef typename Point_set::template Property_map<Type> Pmap;
-    Pmap m_pmap;
-  public:
-    Char_property_printer (const Pmap& pmap) : m_pmap (pmap)
-    {
-
-    }
-    
-    virtual void print(std::ostream& stream, const typename CGAL::Point_set_3<Point,Vector>::Index& index)
-    {
-      if (get_mode(stream) == IO::ASCII)
-        stream << int(get(m_pmap, index));
-      else
-        {
-          Type t = get (m_pmap, index);
-          stream.write (reinterpret_cast<char*>(&t), sizeof(t));
-        }
-    }
-  };
-  
-}
-/// \endcond
-  
 /*!
   
   \ingroup PkgPointSet3IO
@@ -831,124 +999,7 @@ template <typename Point, typename Vector>
 std::ostream& operator<<(std::ostream& os,
                          const CGAL::Point_set_3<Point, Vector>& ps)
 {
-  typedef CGAL::Point_set_3<Point, Vector> Point_set;
-    
-  os << "ply" << std::endl
-     << ((get_mode(os) == IO::BINARY) ? "format binary_little_endian 1.0" : "format ascii 1.0") << std::endl
-     << "comment Generated by the CGAL library" << std::endl
-     << "element vertex " << ps.number_of_points() << std::endl;
-  
-  std::vector<std::string> prop = ps.base().properties();
-  std::vector<internal::Abstract_property_printer<Point, Vector>*> printers;
-  
-  for (std::size_t i = 0; i < prop.size(); ++ i)
-    {
-      if (prop[i] == "index")
-        continue;
-
-      if (prop[i] == "point")
-        {
-          os << "property double x" << std::endl
-             << "property double y" << std::endl
-             << "property double z" << std::endl;
-          printers.push_back (new internal::Property_printer<Point,Vector,Point>(ps.point_map()));
-          continue;
-        }
-      if (prop[i] == "normal")
-        {
-          os << "property double nx" << std::endl
-             << "property double ny" << std::endl
-             << "property double nz" << std::endl;
-          printers.push_back (new internal::Property_printer<Point,Vector,Vector>(ps.normal_map()));
-          continue;
-        }
-      
-      bool okay = false;
-      {
-        typename Point_set::template Property_map<boost::int8_t> pmap;
-        boost::tie (pmap, okay) = ps.template property_map<boost::int8_t>(prop[i]);
-        if (okay)
-          {
-            os << "property char " << prop[i] << std::endl;
-            printers.push_back (new internal::Char_property_printer<Point,Vector,boost::int8_t>(pmap));
-            continue;
-          }
-      }
-      {
-        typename Point_set::template Property_map<boost::uint8_t> pmap;
-        boost::tie (pmap, okay) = ps.template property_map<boost::uint8_t>(prop[i]);
-        if (okay)
-          {
-            os << "property uchar " << prop[i] << std::endl;
-            printers.push_back (new internal::Char_property_printer<Point,Vector,boost::uint8_t>(pmap));
-            continue;
-          }
-      }
-      {
-        typename Point_set::template Property_map<boost::int16_t> pmap;
-        boost::tie (pmap, okay) = ps.template property_map<boost::int16_t>(prop[i]);
-        if (okay)
-          {
-            os << "property short " << prop[i] << std::endl;
-            printers.push_back (new internal::Simple_property_printer<Point,Vector,boost::int16_t>(pmap));
-            continue;
-          }
-      }
-      {
-        typename Point_set::template Property_map<boost::uint16_t> pmap;
-        boost::tie (pmap, okay) = ps.template property_map<boost::uint16_t>(prop[i]);
-        if (okay)
-          {
-            os << "property ushort " << prop[i] << std::endl;
-            printers.push_back (new internal::Simple_property_printer<Point,Vector,boost::uint16_t>(pmap));
-            continue;
-          }
-      }
-      {
-        typename Point_set::template Property_map<boost::int32_t> pmap;
-        boost::tie (pmap, okay) = ps.template property_map<boost::int32_t>(prop[i]);
-        if (okay)
-          {
-            os << "property int " << prop[i] << std::endl;
-            printers.push_back (new internal::Simple_property_printer<Point,Vector,boost::int32_t>(pmap));
-            continue;
-          }
-      }
-      {
-        typename Point_set::template Property_map<float> pmap;
-        boost::tie (pmap, okay) = ps.template property_map<float>(prop[i]);
-        if (okay)
-          {
-            os << "property float " << prop[i] << std::endl;
-            printers.push_back (new internal::Simple_property_printer<Point,Vector,float>(pmap));
-            continue;
-          }
-      }
-      {
-        typename Point_set::template Property_map<double> pmap;
-        boost::tie (pmap, okay) = ps.template property_map<double>(prop[i]);
-        if (okay)
-          {
-            os << "property double " << prop[i] << std::endl;
-            printers.push_back (new internal::Simple_property_printer<Point,Vector,double>(pmap));
-            continue;
-          }
-      }
-    }
-    
-  os << "end_header" << std::endl;  
-
-  for (typename Point_set::const_iterator it = ps.begin(); it != ps.end(); ++ it)
-    {
-      for (std::size_t i = 0; i < printers.size(); ++ i)
-        {
-          printers[i]->print(os, *it);
-          if (get_mode (os) == IO::ASCII)
-            os << " ";
-        }
-      if (get_mode (os) == IO::ASCII)
-        os << std::endl;
-    }
+  write_ply_point_set (os, ps);
   return os;
 }
 
