@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Polynomial/include/CGAL/Polynomial_traits_d.h $
+// $Id: Polynomial_traits_d.h d701f57 2018-06-13T17:10:36+02:00 Mael Rouxel-Labbé
+// SPDX-License-Identifier: LGPL-3.0+
 // 
 //
 // Author(s)     : Michael Hemmer <hemmer@informatik.uni-mainz.de> 
@@ -23,6 +24,8 @@
 #ifndef CGAL_POLYNOMIAL_TRAITS_D_H
 #define CGAL_POLYNOMIAL_TRAITS_D_H
 
+#include <CGAL/disable_warnings.h>
+
 #include <CGAL/basic.h>
 #include <functional>
 #include <list>
@@ -42,7 +45,8 @@
 #include <CGAL/Polynomial/subresultants.h>
 #include <CGAL/Polynomial/sturm_habicht_sequence.h>
 
-#include <boost/iterator/transform_iterator.hpp> 
+#include <CGAL/boost/iterator/transform_iterator.hpp> 
+#include <CGAL/tss.h>
 
 
 #define CGAL_POLYNOMIAL_TRAITS_D_BASE_TYPEDEFS                          \
@@ -119,7 +123,7 @@ public:
 
  
  struct Multivariate_content
-    : public std::unary_function< Polynomial_d , Innermost_coefficient_type >{
+    : public CGAL::cpp98::unary_function< Polynomial_d , Innermost_coefficient_type >{
     Innermost_coefficient_type 
     operator()(const Polynomial_d& p) const {
       typedef Innermost_coefficient_const_iterator IT;
@@ -152,7 +156,7 @@ public:
     
   //       Multivariate_content;
   struct Multivariate_content
-    : public std::unary_function< Polynomial_d , Innermost_coefficient_type >{
+    : public CGAL::cpp98::unary_function< Polynomial_d , Innermost_coefficient_type >{
     Innermost_coefficient_type operator()(const Polynomial_d& p) const {
       if( CGAL::is_zero(p) )
         return Innermost_coefficient_type(0);
@@ -213,7 +217,7 @@ public:
     
   //       Univariate_content
   struct Univariate_content
-    : public std::unary_function< Polynomial_d , Coefficient_type>{
+    : public CGAL::cpp98::unary_function< Polynomial_d , Coefficient_type>{
     Coefficient_type operator()(const Polynomial_d& p) const {
       return p.content();
     }
@@ -283,11 +287,11 @@ public:
   typedef ICoeff Innermost_coefficient_type;
     
   struct Degree 
-    : public std::unary_function< ICoeff , int > {
+    : public CGAL::cpp98::unary_function< ICoeff , int > {
     int operator()(const ICoeff&) const { return 0; }
   };
   struct Total_degree 
-    : public std::unary_function< ICoeff , int > {
+    : public CGAL::cpp98::unary_function< ICoeff , int > {
     int operator()(const ICoeff&) const { return 0; }
   };
     
@@ -305,14 +309,14 @@ public:
   typedef Null_functor  Differentiate;
     
   struct Is_square_free
-    : public std::unary_function< ICoeff, bool > {
+    : public CGAL::cpp98::unary_function< ICoeff, bool > {
     bool operator()( const ICoeff& ) const {
       return true;
     }
   };
     
   struct Make_square_free 
-    : public std::unary_function< ICoeff, ICoeff>{
+    : public CGAL::cpp98::unary_function< ICoeff, ICoeff>{
     ICoeff operator()( const ICoeff& x ) const {
       if (CGAL::is_zero(x)) return x ;
       else  return ICoeff(1);
@@ -325,7 +329,7 @@ public:
   typedef Null_functor  Pseudo_division_quotient;
 
   struct Gcd_up_to_constant_factor 
-    : public std::binary_function< ICoeff, ICoeff, ICoeff >{
+    : public CGAL::cpp98::binary_function< ICoeff, ICoeff, ICoeff >{
     ICoeff operator()(const ICoeff& x, const ICoeff& y) const {
       if (CGAL::is_zero(x) && CGAL::is_zero(y)) 
         return ICoeff(0);
@@ -337,7 +341,7 @@ public:
   typedef Null_functor Integral_division_up_to_constant_factor;
 
   struct Univariate_content_up_to_constant_factor
-    : public std::unary_function< ICoeff, ICoeff >{
+    : public CGAL::cpp98::unary_function< ICoeff, ICoeff >{
     ICoeff operator()(const ICoeff& ) const {
       // TODO: Why not return 0 if argument is 0 ? 
       return ICoeff(1);
@@ -350,7 +354,7 @@ public:
   typedef Null_functor  Evaluate_homogeneous;
     
   struct Innermost_leading_coefficient
-    :public std::unary_function <ICoeff, ICoeff>{
+    :public CGAL::cpp98::unary_function <ICoeff, ICoeff>{
     const ICoeff& operator()(const ICoeff& x){return x;}
   };
     
@@ -364,7 +368,7 @@ public:
   };
   
   struct Get_innermost_coefficient 
-    : public std::binary_function< ICoeff, Polynomial_d, Exponent_vector > {
+    : public CGAL::cpp98::binary_function< ICoeff, Polynomial_d, Exponent_vector > {
     const ICoeff& operator()( const Polynomial_d& p, Exponent_vector ) {
       return p;
     }
@@ -471,7 +475,7 @@ private:
   // coeff type has no comparison operators available.
 private:
   struct Compare_exponents_coeff_pair 
-    : public std::binary_function< 
+    : public CGAL::cpp98::binary_function<
        std::pair< Exponent_vector, Innermost_coefficient_type >,
        std::pair< Exponent_vector, Innermost_coefficient_type >,
        bool > 
@@ -697,10 +701,10 @@ public:
 
   // Get_coefficient;
   struct Get_coefficient 
-    : public std::binary_function<Polynomial_d, int, Coefficient_type > {
+    : public CGAL::cpp98::binary_function<Polynomial_d, int, Coefficient_type > {
         
     const Coefficient_type& operator()( const Polynomial_d& p, int i) const {
-      static const Coefficient_type zero =  Coefficient_type(0);
+      CGAL_STATIC_THREAD_LOCAL_VARIABLE(Coefficient_type, zero, 0);
       CGAL_precondition( i >= 0 );
       typename PT::Degree degree;
       if( i >  degree(p) )
@@ -711,8 +715,9 @@ public:
     
   //     Get_innermost_coefficient;
   struct Get_innermost_coefficient
-    : public 
-    std::binary_function< Polynomial_d, Exponent_vector, Innermost_coefficient_type >
+    : public CGAL::cpp98::binary_function< Polynomial_d,
+                                           Exponent_vector,
+                                           Innermost_coefficient_type >
   {
         
     const Innermost_coefficient_type& 
@@ -725,7 +730,9 @@ public:
       return gic( gc( p, exponent ), ev ); 
     }; 
   };
-     
+
+  typedef CGAL::internal::Monomial_representation<Polynomial_d> Monomial_representation;
+
   // Swap variable x_i with x_j
   struct Swap {
     typedef Polynomial_d        result_type;  
@@ -830,7 +837,7 @@ public:
   typedef CGAL::internal::Degree<Polynomial_d> Degree; 
 
   //       Total_degree;
-  struct Total_degree : public std::unary_function< Polynomial_d , int >{
+  struct Total_degree : public CGAL::cpp98::unary_function< Polynomial_d , int >{
     int operator()(const Polynomial_d& p) const {
       typedef Polynomial_traits_d<Coefficient_type> COEFF_POLY_TRAITS;
       typename COEFF_POLY_TRAITS::Total_degree total_degree;
@@ -848,7 +855,7 @@ public:
 
   //       Leading_coefficient;
   struct Leading_coefficient 
-    : public std::unary_function< Polynomial_d , Coefficient_type>{
+    : public CGAL::cpp98::unary_function< Polynomial_d , Coefficient_type>{
     const Coefficient_type& operator()(const Polynomial_d& p) const {
       return p.lcoeff();
     }
@@ -856,7 +863,7 @@ public:
     
   //       Innermost_leading_coefficient;
   struct Innermost_leading_coefficient 
-    : public std::unary_function< Polynomial_d , Innermost_coefficient_type>{
+    : public CGAL::cpp98::unary_function< Polynomial_d , Innermost_coefficient_type>{
     const Innermost_coefficient_type& 
     operator()(const Polynomial_d& p) const {
       typename PTC::Innermost_leading_coefficient ilcoeff;
@@ -868,7 +875,7 @@ public:
   
   //return a canonical representative of all constant multiples. 
   struct Canonicalize
-    : public std::unary_function<Polynomial_d, Polynomial_d>{
+    : public CGAL::cpp98::unary_function<Polynomial_d, Polynomial_d>{
  
   private: 
     inline Polynomial_d canonicalize_(Polynomial_d p, CGAL::Tag_true) const 
@@ -910,7 +917,7 @@ public:
 
   //       Differentiate;
   struct Differentiate 
-    : public std::unary_function<Polynomial_d, Polynomial_d>{
+    : public CGAL::cpp98::unary_function<Polynomial_d, Polynomial_d>{
     Polynomial_d
     operator()(Polynomial_d p, int i = (d-1)) const {
       if (i == (d-1) ){
@@ -927,7 +934,7 @@ public:
 
   // Evaluate;
   struct Evaluate
-    :public std::binary_function<Polynomial_d,Coefficient_type,Coefficient_type>{
+    :public CGAL::cpp98::binary_function<Polynomial_d,Coefficient_type,Coefficient_type>{
     // Evaluate with respect to one variable 
     Coefficient_type
     operator()(const Polynomial_d& p, const Coefficient_type& x) const {
@@ -1044,7 +1051,7 @@ public:
  
 
 struct Construct_coefficient_const_iterator_range                                       
-     : public std::unary_function< Polynomial_d, 
+     : public CGAL::cpp98::unary_function< Polynomial_d, 
                                   Coefficient_const_iterator_range> {
     Coefficient_const_iterator_range      
     operator () (const Polynomial_d& p) const {                                      
@@ -1053,7 +1060,7 @@ struct Construct_coefficient_const_iterator_range
 };        
                                        
 struct Construct_innermost_coefficient_const_iterator_range                                       
-   : public std::unary_function< Polynomial_d, 
+   : public CGAL::cpp98::unary_function< Polynomial_d, 
                                  Innermost_coefficient_const_iterator_range> {
    Innermost_coefficient_const_iterator_range      
    operator () (const Polynomial_d& p) const {                                      
@@ -1064,7 +1071,7 @@ struct Construct_innermost_coefficient_const_iterator_range
 };                           
   
   struct Is_square_free 
-    : public std::unary_function< Polynomial_d, bool >{
+    : public CGAL::cpp98::unary_function< Polynomial_d, bool >{
     bool operator()( const Polynomial_d& p ) const {
       if( !internal::may_have_multiple_factor( p ) )
         return true;
@@ -1089,7 +1096,7 @@ struct Construct_innermost_coefficient_const_iterator_range
 
                    
   struct Make_square_free 
-    : public std::unary_function< Polynomial_d, Polynomial_d >{
+    : public CGAL::cpp98::unary_function< Polynomial_d, Polynomial_d >{
     Polynomial_d
     operator()(const Polynomial_d& p) const {
       if (CGAL::is_zero(p)) return p;
@@ -1124,7 +1131,7 @@ struct Construct_innermost_coefficient_const_iterator_range
   };
     
   struct Pseudo_division_quotient
-    :public std::binary_function<Polynomial_d, Polynomial_d, Polynomial_d> {
+    :public CGAL::cpp98::binary_function<Polynomial_d, Polynomial_d, Polynomial_d> {
         
     Polynomial_d
     operator()(const Polynomial_d& f, const Polynomial_d& g) const {
@@ -1136,7 +1143,7 @@ struct Construct_innermost_coefficient_const_iterator_range
   };
 
   struct Pseudo_division_remainder
-    :public std::binary_function<Polynomial_d, Polynomial_d, Polynomial_d> {
+    :public CGAL::cpp98::binary_function<Polynomial_d, Polynomial_d, Polynomial_d> {
         
     Polynomial_d
     operator()(const Polynomial_d& f, const Polynomial_d& g) const {
@@ -1148,7 +1155,7 @@ struct Construct_innermost_coefficient_const_iterator_range
   };
     
   struct Gcd_up_to_constant_factor
-    :public std::binary_function<Polynomial_d, Polynomial_d, Polynomial_d> {
+    :public CGAL::cpp98::binary_function<Polynomial_d, Polynomial_d, Polynomial_d> {
     Polynomial_d
     operator()(const Polynomial_d& p, const Polynomial_d& q) const {
       if(p==q) return CGAL::canonicalize(p); 
@@ -1165,7 +1172,7 @@ struct Construct_innermost_coefficient_const_iterator_range
   };
     
   struct Integral_division_up_to_constant_factor
-    :public std::binary_function<Polynomial_d, Polynomial_d, Polynomial_d> {
+    :public CGAL::cpp98::binary_function<Polynomial_d, Polynomial_d, Polynomial_d> {
    
   
     
@@ -1194,7 +1201,7 @@ struct Construct_innermost_coefficient_const_iterator_range
   };
     
   struct Univariate_content_up_to_constant_factor
-    :public std::unary_function<Polynomial_d, Coefficient_type> {
+    :public CGAL::cpp98::unary_function<Polynomial_d, Coefficient_type> {
     Coefficient_type
     operator()(const Polynomial_d& p) const {
       typename PTC::Gcd_up_to_constant_factor gcd_utcf;
@@ -1269,7 +1276,7 @@ struct Construct_innermost_coefficient_const_iterator_range
   };
 
   struct Shift
-    : public std::binary_function< Polynomial_d,int,Polynomial_d >{
+    : public CGAL::cpp98::binary_function< Polynomial_d,int,Polynomial_d >{
     
     Polynomial_d 
     operator()(const Polynomial_d& p, int e, int i = (d-1)) const {
@@ -1287,7 +1294,7 @@ struct Construct_innermost_coefficient_const_iterator_range
   };
     
   struct Negate
-    : public std::unary_function< Polynomial_d, Polynomial_d >{
+    : public CGAL::cpp98::unary_function< Polynomial_d, Polynomial_d >{
         
     Polynomial_d operator()(const Polynomial_d& p, int i = (d-1)) const {
       Construct_polynomial construct; 
@@ -1305,7 +1312,7 @@ struct Construct_innermost_coefficient_const_iterator_range
   };
 
   struct Invert
-    : public std::unary_function< Polynomial_d , Polynomial_d >{
+    : public CGAL::cpp98::unary_function< Polynomial_d , Polynomial_d >{
     Polynomial_d operator()(Polynomial_d p, int i = (PT::d-1)) const {
       if (i == (d-1)){
         p.reversal(); 
@@ -1319,7 +1326,7 @@ struct Construct_innermost_coefficient_const_iterator_range
   };
 
   struct Translate
-    : public std::binary_function< Polynomial_d , Innermost_coefficient_type, 
+    : public CGAL::cpp98::binary_function< Polynomial_d , Innermost_coefficient_type,
                                    Polynomial_d >{
     Polynomial_d
     operator()(
@@ -1362,10 +1369,11 @@ struct Construct_innermost_coefficient_const_iterator_range
     }
   };
 
-  struct Scale 
-    : public 
-    std::binary_function< Polynomial_d, Innermost_coefficient_type, Polynomial_d > {
-        
+  struct Scale
+    : public CGAL::cpp98::binary_function< Polynomial_d,
+                                           Innermost_coefficient_type,
+                                           Polynomial_d >
+  {
     Polynomial_d operator()( Polynomial_d p, const Innermost_coefficient_type& c,
         int i = (PT::d-1) ) const  {
       CGAL_precondition( i <= d-1 );
@@ -1410,7 +1418,7 @@ struct Construct_innermost_coefficient_const_iterator_range
   };
 
   struct Resultant
-    : public std::binary_function<Polynomial_d, Polynomial_d, Coefficient_type>{
+    : public CGAL::cpp98::binary_function<Polynomial_d, Polynomial_d, Coefficient_type>{
         
     Coefficient_type
     operator()(
@@ -1533,10 +1541,6 @@ struct Construct_innermost_coefficient_const_iterator_range
   };
 
 
-  typedef
-  CGAL::internal::Monomial_representation<Polynomial_d> 
-  Monomial_representation;
-
   // returns the Exponten_vector of the innermost leading coefficient 
   struct Degree_vector{
     typedef Exponent_vector           result_type;
@@ -1713,4 +1717,6 @@ public:
 
 } //namespace CGAL
 
+#include <CGAL/enable_warnings.h>
+
 #endif // CGAL_POLYNOMIAL_TRAITS_D_H
